@@ -73,7 +73,7 @@ static void sbr_neg_odd_64_c(int *x)
 {
     int i;
     for (i = 1; i < 64; i += 2)
-        x[i] = -(unsigned)x[i];
+        x[i] = -x[i];
 }
 
 static void sbr_qmf_pre_shuffle_c(int *z)
@@ -100,8 +100,8 @@ static void sbr_qmf_deint_neg_c(int *v, const int *src)
 {
     int i;
     for (i = 0; i < 32; i++) {
-        v[     i] = (int)(0x10U + src[63 - 2*i    ]) >> 5;
-        v[63 - i] = (int)(0x10U - src[63 - 2*i - 1]) >> 5;
+        v[     i] = ( src[63 - 2*i    ] + 0x10) >> 5;
+        v[63 - i] = (-src[63 - 2*i - 1] + 0x10) >> 5;
     }
 }
 
@@ -136,19 +136,19 @@ static av_always_inline void autocorrelate(const int x[40][2], SoftFloat phi[3][
 
     if (lag) {
         for (i = 1; i < 38; i++) {
-            accu_re += (uint64_t)x[i][0] * x[i+lag][0];
-            accu_re += (uint64_t)x[i][1] * x[i+lag][1];
-            accu_im += (uint64_t)x[i][0] * x[i+lag][1];
-            accu_im -= (uint64_t)x[i][1] * x[i+lag][0];
+            accu_re += (int64_t)x[i][0] * x[i+lag][0];
+            accu_re += (int64_t)x[i][1] * x[i+lag][1];
+            accu_im += (int64_t)x[i][0] * x[i+lag][1];
+            accu_im -= (int64_t)x[i][1] * x[i+lag][0];
         }
 
         real_sum = accu_re;
         imag_sum = accu_im;
 
-        accu_re += (uint64_t)x[ 0][0] * x[lag][0];
-        accu_re += (uint64_t)x[ 0][1] * x[lag][1];
-        accu_im += (uint64_t)x[ 0][0] * x[lag][1];
-        accu_im -= (uint64_t)x[ 0][1] * x[lag][0];
+        accu_re += (int64_t)x[ 0][0] * x[lag][0];
+        accu_re += (int64_t)x[ 0][1] * x[lag][1];
+        accu_im += (int64_t)x[ 0][0] * x[lag][1];
+        accu_im -= (int64_t)x[ 0][1] * x[lag][0];
 
         phi[2-lag][1][0] = autocorr_calc(accu_re);
         phi[2-lag][1][1] = autocorr_calc(accu_im);
@@ -156,28 +156,28 @@ static av_always_inline void autocorrelate(const int x[40][2], SoftFloat phi[3][
         if (lag == 1) {
             accu_re = real_sum;
             accu_im = imag_sum;
-            accu_re += (uint64_t)x[38][0] * x[39][0];
-            accu_re += (uint64_t)x[38][1] * x[39][1];
-            accu_im += (uint64_t)x[38][0] * x[39][1];
-            accu_im -= (uint64_t)x[38][1] * x[39][0];
+            accu_re += (int64_t)x[38][0] * x[39][0];
+            accu_re += (int64_t)x[38][1] * x[39][1];
+            accu_im += (int64_t)x[38][0] * x[39][1];
+            accu_im -= (int64_t)x[38][1] * x[39][0];
 
             phi[0][0][0] = autocorr_calc(accu_re);
             phi[0][0][1] = autocorr_calc(accu_im);
         }
     } else {
         for (i = 1; i < 38; i++) {
-            accu_re += (uint64_t)x[i][0] * x[i][0];
-            accu_re += (uint64_t)x[i][1] * x[i][1];
+            accu_re += (int64_t)x[i][0] * x[i][0];
+            accu_re += (int64_t)x[i][1] * x[i][1];
         }
         real_sum = accu_re;
-        accu_re += (uint64_t)x[ 0][0] * x[ 0][0];
-        accu_re += (uint64_t)x[ 0][1] * x[ 0][1];
+        accu_re += (int64_t)x[ 0][0] * x[ 0][0];
+        accu_re += (int64_t)x[ 0][1] * x[ 0][1];
 
         phi[2][1][0] = autocorr_calc(accu_re);
 
         accu_re = real_sum;
-        accu_re += (uint64_t)x[38][0] * x[38][0];
-        accu_re += (uint64_t)x[38][1] * x[38][1];
+        accu_re += (int64_t)x[38][0] * x[38][0];
+        accu_re += (int64_t)x[38][1] * x[38][1];
 
         phi[1][0][0] = autocorr_calc(accu_re);
     }
@@ -233,14 +233,12 @@ static void sbr_hf_g_filt_c(int (*Y)[2], const int (*X_high)[40][2],
     int64_t accu;
 
     for (m = 0; m < m_max; m++) {
-        if (22 - g_filt[m].exp < 61) {
-            int64_t r = 1LL << (22-g_filt[m].exp);
-            accu = (int64_t)X_high[m][ixh][0] * ((g_filt[m].mant + 0x40)>>7);
-            Y[m][0] = (int)((accu + r) >> (23-g_filt[m].exp));
+        int64_t r = 1LL << (22-g_filt[m].exp);
+        accu = (int64_t)X_high[m][ixh][0] * ((g_filt[m].mant + 0x40)>>7);
+        Y[m][0] = (int)((accu + r) >> (23-g_filt[m].exp));
 
-            accu = (int64_t)X_high[m][ixh][1] * ((g_filt[m].mant + 0x40)>>7);
-            Y[m][1] = (int)((accu + r) >> (23-g_filt[m].exp));
-        }
+        accu = (int64_t)X_high[m][ixh][1] * ((g_filt[m].mant + 0x40)>>7);
+        Y[m][1] = (int)((accu + r) >> (23-g_filt[m].exp));
     }
 }
 
@@ -255,8 +253,8 @@ static av_always_inline int sbr_hf_apply_noise(int (*Y)[2],
     int m;
 
     for (m = 0; m < m_max; m++) {
-        unsigned y0 = Y[m][0];
-        unsigned y1 = Y[m][1];
+        int y0 = Y[m][0];
+        int y1 = Y[m][1];
         noise = (noise + 1) & 0x1ff;
         if (s_m[m].mant) {
             int shift, round;
